/**
 * @file
 * The glossary link UI.
 */

import { Plugin } from "ckeditor5/src/core";
import { ButtonView } from "ckeditor5/src/ui";
import { ContextualBalloon } from "ckeditor5/src/ui";
import GlossaryLinkFormView from "./glossarylinkformview";
import glossaryLinkIcon from "../theme/icons/glossary-link.svg";

// Access jQuery from global scope
const $ = jQuery;

/**
 * The glossary link UI plugin.
 */
export default class GlossaryLinkUI extends Plugin {
	/**
	 * @inheritdoc
	 */
	static get requires() {
		return [ContextualBalloon];
	}

	/**
	 * @inheritdoc
	 */
	static get pluginName() {
		return "GlossaryLinkUI";
	}

	/**
	 * @inheritdoc
	 */
	init() {
		this._createToolbarButton();
		this._createFormView();
	}

	/**
	 * Creates the toolbar button.
	 */
	_createToolbarButton() {
		const editor = this.editor;
		const command = editor.commands.get("glossaryLink");

		editor.ui.componentFactory.add("glossaryLink", (locale) => {
			const button = new ButtonView(locale);

			button.set({
				label: "Glossary Link",
				icon: glossaryLinkIcon,
				tooltip: true,
				isToggleable: true,
			});

			button.bind("isOn").to(command, "value", (value) => !!value);
			button.bind("isEnabled").to(command, "isEnabled");

			button.on("execute", () => {
				this._showForm();
			});

			return button;
		});
	}

	/**
	 * Creates the form view.
	 */
	_createFormView() {
		const editor = this.editor;
		const options = editor.config.get("ckeditorTaxonomyGlossary") || {};

		this.formView = new GlossaryLinkFormView(editor.locale, options);

		this.formView.on("submit", () => {
			const $input = $(this.formView.termIdInputView.fieldView.element);
			const termId =
				$input.data("glossary-id") ||
				this.formView.termIdInputView.fieldView.element.value;

			if (termId) {
				// Validate that termId is numeric
				if (!/^\d+$/.test(termId)) {
					// Show error message for invalid term ID
					alert(
						"Please select a valid glossary term from the autocomplete suggestions.",
					);
					return;
				}
				editor.execute("glossaryLink", termId);
			}
			this._hideForm();
		});

		this.formView.on("cancel", () => {
			this._hideForm();
		});

		this.formView.on("removeLink", () => {
			editor.execute("glossaryLink", null);
			this._hideForm();
		});
	}

	/**
	 * Shows the form.
	 */
	async _showForm() {
		const editor = this.editor;
		const balloon = this.editor.plugins.get(ContextualBalloon);

		if (balloon.hasView(this.formView)) {
			return;
		}

		// Initialize form values properly
		await this._initializeFormValues();

		// Add balloon close listeners
		this._addBalloonCloseListeners();

		balloon.add({
			view: this.formView,
			position: this._getBalloonPositionData(),
		});

		this.formView.focus();

		// Initialize autocomplete after the form is shown
		this._initializeAutocomplete();
	}

	/**
	 * Initializes form values from the current command state.
	 */
	async _initializeFormValues() {
		const command = this.editor.commands.get("glossaryLink");
		const $input = $(this.formView.termIdInputView.fieldView.element);

		// Safely set the form value using CKEditor 5 API
		if (
			command.value &&
			this.formView.termIdInputView &&
			this.formView.termIdInputView.fieldView
		) {
			// For existing links, fetch the term name and display "Term Name (ID)"
			try {
				const response = await fetch(`/glossary/term/${command.value}`);
				if (response.ok) {
					const termData = await response.json();
					const displayValue = `${termData.name} (${termData.id})`;
					this.formView.termIdInputView.fieldView.set("value", displayValue);
					$input.data("glossary-id", termData.id);
				} else {
					// Fallback to just the ID if fetch fails
					this.formView.termIdInputView.fieldView.set("value", command.value);
				}
			} catch (error) {
				console.warn("Failed to fetch term name:", error);
				// Fallback to just the ID if fetch fails
				this.formView.termIdInputView.fieldView.set("value", command.value);
			}
		} else {
			// Clear the field if no value
			this.formView.termIdInputView.fieldView.set("value", "");
			$input.removeData("glossary-id");
		}

		// Set editing mode based on whether we have an existing link
		this.formView.setEditingMode(!!command.value);
	}

	/**
	 * Adds balloon close listeners.
	 */
	_addBalloonCloseListeners() {
		// Bind methods to preserve context
		this._boundOnEscapeKey = this._onEscapeKey.bind(this);
		this._boundOnClickOutside = this._onClickOutside.bind(this);

		// Add listeners with a slight delay to prevent immediate closure
		setTimeout(() => {
			document.addEventListener("keydown", this._boundOnEscapeKey);
			document.addEventListener("click", this._boundOnClickOutside);
		}, 100);
	}

	/**
	 * Removes balloon close listeners.
	 */
	_removeBalloonCloseListeners() {
		if (this._boundOnEscapeKey) {
			document.removeEventListener("keydown", this._boundOnEscapeKey);
			this._boundOnEscapeKey = null;
		}
		if (this._boundOnClickOutside) {
			document.removeEventListener("click", this._boundOnClickOutside);
			this._boundOnClickOutside = null;
		}
	}

	/**
	 * Handles escape key press.
	 */
	_onEscapeKey(event) {
		if (event.key === "Escape" && this._isBalloonVisible()) {
			event.preventDefault();
			event.stopPropagation();
			this._hideForm();
		}
	}

	/**
	 * Handles clicks outside the balloon.
	 */
	_onClickOutside(event) {
		if (!this._isBalloonVisible()) {
			return;
		}

		const balloon = this.editor.plugins.get(ContextualBalloon);
		const balloonElement = balloon.view.element;

		// Check if click is inside balloon
		if (balloonElement?.contains(event.target)) {
			return;
		}

		// Check if click is on the glossary button (prevent closure when opening)
		const glossaryButton = document.querySelector(
			'[data-cke-tooltip-text="Glossary Link"]',
		);
		if (
			glossaryButton &&
			(glossaryButton === event.target || glossaryButton.contains(event.target))
		) {
			return;
		}

		// Check if click is on the autocomplete dropdown
		const autocompleteDropdown = document.querySelector(".ui-autocomplete");
		if (autocompleteDropdown?.contains(event.target)) {
			return;
		}

		// Check if click is on any autocomplete item
		const autocompleteItem = event.target.closest(
			".glossary-autocomplete-item, .glossary-autocomplete-wrapper, .ui-menu-item",
		);
		if (autocompleteItem) {
			return;
		}

		this._hideForm();
	}

	/**
	 * Checks if the balloon is currently visible.
	 */
	_isBalloonVisible() {
		const balloon = this.editor.plugins.get(ContextualBalloon);
		return balloon?.hasView(this.formView);
	}

	/**
	 * Initializes autocomplete on the term ID input.
	 */
	_initializeAutocomplete() {
		const input = this.formView.termIdInputView.fieldView.element;
		const $input = $(input);

		// Check if autocomplete is already initialized
		if ($input.hasClass("ui-autocomplete-input")) {
			return;
		}

		// Get the balloon element to append autocomplete dropdown to it
		const balloon = this.editor.plugins.get(ContextualBalloon);
		const balloonElement = balloon.view.element;

		$input
			.autocomplete({
				appendTo: balloonElement, // Append autocomplete dropdown to balloon
				source: (request, response) => {
					$.ajax({
						url: `/glossary/autocomplete/${encodeURIComponent(request.term)}`,
						dataType: "json",
						success: (data) => {
							response(
								data.map((item) => ({
									label: item.label,
									value: item.id,
									description: item.description,
									langcode: item.langcode,
									language_name: item.language_name,
								})),
							);
						},
						error: (xhr) => {
							console.error(
								"Glossary autocomplete failed:",
								xhr.status,
								xhr.statusText,
							);
							if (xhr.status === 403) {
								console.warn(
									"Access denied to glossary autocomplete. Please check permissions.",
								);
							}
							response([]);
						},
					});
				},
				minLength: 2,
				select: (event, ui) => {
					// Store the ID in a data attribute and display the label with ID
					$input.val(`${ui.item.label} (${ui.item.value})`);
					$input.data("glossary-id", ui.item.value);
					// Prevent the balloon from closing when selecting an item
					event.stopPropagation();
					return false;
				},
				focus: (event, ui) => {
					return false;
				},
			})
			.autocomplete("instance")._renderItem = (ul, item) => {
			const $li = $("<li>").addClass("glossary-autocomplete-item");
			const $wrapper = $("<div>").addClass("glossary-autocomplete-wrapper");
			const $header = $("<div>").addClass("glossary-autocomplete-header");

			// Add language indicator if available (exclude 'und' and 'zxx' for undefined/not applicable languages)
			if (
				Object.prototype.hasOwnProperty.call(item, "langcode") &&
				item.langcode &&
				!["und", "zxx"].includes(item.langcode.toLowerCase())
			) {
				$header.append(
					$("<span>")
						.text(item.langcode.toUpperCase())
						.addClass("glossary-autocomplete-language"),
				);
			}

			$header.append(
				$("<span>").addClass("glossary-autocomplete-label").text(item.label),
			);

			$wrapper.append($header);

			if (item.description) {
				$wrapper.append(
					$("<span>")
						.addClass("glossary-autocomplete-description")
						.text(item.description),
				);
			}

			return $li.append($wrapper).appendTo(ul);
		};
	}

	/**
	 * Hides the form.
	 */
	_hideForm() {
		const balloon = this.editor.plugins.get(ContextualBalloon);
		balloon.remove(this.formView);
		this.editor.editing.view.focus();

		// Remove balloon close listeners
		this._removeBalloonCloseListeners();
	}

	/**
	 * Returns the balloon position data.
	 */
	_getBalloonPositionData() {
		const view = this.editor.editing.view;
		const viewDocument = view.document;
		const selection = viewDocument.selection;

		return {
			target: () => view.domConverter.viewRangeToDom(selection.getFirstRange()),
		};
	}
}
