<?php

declare(strict_types=1);

namespace Drupal\ckeditor_taxonomy_glossary\Controller;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\taxonomy\TermInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for glossary autocomplete and term description endpoints.
 */
final class GlossaryAutocompleteController extends ControllerBase {

  /**
   * Returns autocomplete suggestions for glossary terms.
   *
   * @param string $text
   *   The text to search for.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A JSON response with matching terms.
   */
  public function autocomplete(string $text): CacheableJsonResponse {
    $matches = [];

    // Only search if text is long enough
    if (strlen($text) >= 2) {
      /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
      $term_storage = $this->entityTypeManager()->getStorage('taxonomy_term');

      $query = $term_storage->getQuery()
        ->accessCheck(TRUE)
        ->condition('vid', 'glossary')
        ->condition('name', $text, 'CONTAINS')
        ->range(0, 20)
        ->sort('name');

      // Include all languages for multilingual support
      // Users can see language indicators in the autocomplete dropdown

      $tids = $query->execute();

      if (!empty($tids)) {
        /** @var \Drupal\taxonomy\TermInterface[] $terms */
        $terms = $term_storage->loadMultiple($tids);

        foreach ($terms as $term) {
          $description = '';
          if ($term->hasField('description') && !$term->get('description')->isEmpty()) {
            $description = strip_tags($term->get('description')->value);
            $description = mb_substr($description, 0, 100) . (mb_strlen($description) > 100 ? '...' : '');
          }

          $matches[] = [
            'id'          => $term->id(),
            'label'       => $term->getName(),
            'description' => $description,
            'langcode'    => $term->language()->getId(),
            'language_name' => $term->language()->getName(),
          ];
        }
      }
    }

    return $this->createCachedAutocompleteResponse($matches);
  }

  /**
   * Creates a cached autocomplete response with proper cache metadata.
   *
   * @param array $matches
   *   The autocomplete matches array.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A cached JSON response.
   */
  private function createCachedAutocompleteResponse(array $matches): CacheableJsonResponse {
    $response = new CacheableJsonResponse($matches);
    $cache_metadata = new CacheableMetadata();
    $cache_metadata->addCacheTags(['taxonomy_term_list:glossary']);
    $cache_metadata->addCacheContexts(['url.path']);
    $response->addCacheableDependency($cache_metadata);

    // Apply cache duration setting from configuration
    $config = $this->config('ckeditor_taxonomy_glossary.settings');
    $cache_duration = $config->get('cache_duration') ?? '3600';

    if ($cache_duration === '0') {
      // No cache - set headers to prevent caching
      $response->setMaxAge(0);
      $response->setSharedMaxAge(0);
      $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
      $response->headers->set('Pragma', 'no-cache');
      $response->headers->set('Expires', '0');
    } else {
      // Set cache duration
      $cache_seconds = (int) $cache_duration;
      $response->setMaxAge($cache_seconds);
      $response->setSharedMaxAge($cache_seconds);
    }

    return $response;
  }

  /**
   * Returns the description for a specific glossary term.
   *
   * @param int $tid
   *   The term ID.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A JSON response with the term description.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If the term is not found or not in the glossary vocabulary.
   */
  public function termDescription(int $tid): CacheableJsonResponse {
    /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
    $term_storage = $this->entityTypeManager()->getStorage('taxonomy_term');

    /** @var \Drupal\taxonomy\TermInterface|null $term */
    $term = $term_storage->load($tid);

    if (!$term instanceof TermInterface || $term->bundle() !== 'glossary') {
      throw new NotFoundHttpException();
    }

    // Use translated version if available
    $current_language = $this->languageManager()->getCurrentLanguage()->getId();
    if ($term->hasTranslation($current_language)) {
      $term = $term->getTranslation($current_language);
    }

    $description = '';
    if ($term->hasField('description') && !$term->get('description')->isEmpty()) {
      $description = $term->get('description')->processed;
    }

    $data = [
      'id'          => $term->id(),
      'name'        => $term->getName(),
      'description' => $description,
      'langcode'    => $term->language()->getId(),
      'language_name' => $term->language()->getName(),
    ];

    $response = new CacheableJsonResponse($data);
    $response->addCacheableDependency($term);

    // Apply cache duration setting from configuration
    $config = $this->config('ckeditor_taxonomy_glossary.settings');
    $cache_duration = $config->get('cache_duration') ?? '3600';

    if ($cache_duration === '0') {
      // No cache - set headers to prevent caching
      $response->setMaxAge(0);
      $response->setSharedMaxAge(0);
      $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
      $response->headers->set('Pragma', 'no-cache');
      $response->headers->set('Expires', '0');
    } else {
      // Set cache duration
      $cache_seconds = (int) $cache_duration;
      $response->setMaxAge($cache_seconds);
      $response->setSharedMaxAge($cache_seconds);
    }

    return $response;
  }

  /**
   * Returns basic term information for a specific glossary term.
   *
   * @param int $tid
   *   The term ID.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A JSON response with basic term information.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If the term is not found or not in the glossary vocabulary.
   */
  public function termInfo(int $tid): CacheableJsonResponse {
    /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
    $term_storage = $this->entityTypeManager()->getStorage('taxonomy_term');

    /** @var \Drupal\taxonomy\TermInterface|null $term */
    $term = $term_storage->load($tid);

    if (!$term instanceof TermInterface || $term->bundle() !== 'glossary') {
      throw new NotFoundHttpException();
    }

    // Use translated version if available
    $current_language = $this->languageManager()->getCurrentLanguage()->getId();
    if ($term->hasTranslation($current_language)) {
      $term = $term->getTranslation($current_language);
    }

    $data = [
      'id'   => $term->id(),
      'name' => $term->getName(),
      'langcode' => $term->language()->getId(),
      'language_name' => $term->language()->getName(),
    ];

    $response = new CacheableJsonResponse($data);
    $response->addCacheableDependency($term);

    // Apply cache duration setting from configuration
    $config = $this->config('ckeditor_taxonomy_glossary.settings');
    $cache_duration = $config->get('cache_duration') ?? '3600';

    if ($cache_duration === '0') {
      // No cache - set headers to prevent caching
      $response->setMaxAge(0);
      $response->setSharedMaxAge(0);
      $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
      $response->headers->set('Pragma', 'no-cache');
      $response->headers->set('Expires', '0');
    } else {
      // Set cache duration
      $cache_seconds = (int) $cache_duration;
      $response->setMaxAge($cache_seconds);
      $response->setSharedMaxAge($cache_seconds);
    }

    return $response;
  }

  /**
   * Returns autocomplete suggestions for glossary terms in a specific language.
   *
   * @param string $text
   *   The text to search for.
   * @param string $langcode
   *   The language code to filter by.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A JSON response with matching terms.
   */
  public function autocompleteByLanguage(string $text, string $langcode): CacheableJsonResponse {
    $matches = [];

    // Only search if text is long enough
    if (strlen($text) >= 2) {
      /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
      $term_storage = $this->entityTypeManager()->getStorage('taxonomy_term');

      $query = $term_storage->getQuery()
        ->accessCheck(TRUE)
        ->condition('vid', 'glossary')
        ->condition('name', $text, 'CONTAINS')
        ->condition('langcode', $langcode)
        ->range(0, 10)
        ->sort('name');

      $tids = $query->execute();

      if (!empty($tids)) {
        /** @var \Drupal\taxonomy\TermInterface[] $terms */
        $terms = $term_storage->loadMultiple($tids);

        foreach ($terms as $term) {
          $description = '';
          if ($term->hasField('description') && !$term->get('description')->isEmpty()) {
            $description = strip_tags($term->get('description')->value);
            $description = mb_substr($description, 0, 100) . (mb_strlen($description) > 100 ? '...' : '');
          }

          $matches[] = [
            'id'            => $term->id(),
            'label'         => $term->getName(),
            'description'   => $description,
            'langcode'      => $term->language()->getId(),
            'language_name' => $term->language()->getName(),
          ];
        }
      }
    }

    return $this->createCachedAutocompleteResponse($matches);
  }

  /**
   * Returns the description for a specific glossary term in a specific language.
   *
   * @param int $tid
   *   The term ID.
   * @param string $langcode
   *   The language code.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   A JSON response with the term description.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If the term is not found or not in the glossary vocabulary.
   */
  public function termDescriptionByLanguage(int $tid, string $langcode): CacheableJsonResponse {
    /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
    $term_storage = $this->entityTypeManager()->getStorage('taxonomy_term');

    /** @var \Drupal\taxonomy\TermInterface|null $term */
    $term = $term_storage->load($tid);

    if (!$term instanceof TermInterface || $term->bundle() !== 'glossary') {
      throw new NotFoundHttpException();
    }

    // Use specific language version if available
    if ($term->hasTranslation($langcode)) {
      $term = $term->getTranslation($langcode);
    } else {
      // If translation doesn't exist, throw not found
      throw new NotFoundHttpException();
    }

    $description = '';
    if ($term->hasField('description') && !$term->get('description')->isEmpty()) {
      $description = $term->get('description')->processed;
    }

    $data = [
      'id'            => $term->id(),
      'name'          => $term->getName(),
      'description'   => $description,
      'langcode'      => $term->language()->getId(),
      'language_name' => $term->language()->getName(),
    ];

    $response = new CacheableJsonResponse($data);
    $response->addCacheableDependency($term);

    // Apply cache duration setting from configuration
    $config = $this->config('ckeditor_taxonomy_glossary.settings');
    $cache_duration = $config->get('cache_duration') ?? '3600';

    if ($cache_duration === '0') {
      // No cache - set headers to prevent caching
      $response->setMaxAge(0);
      $response->setSharedMaxAge(0);
      $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
      $response->headers->set('Pragma', 'no-cache');
      $response->headers->set('Expires', '0');
    } else {
      // Set cache duration
      $cache_seconds = (int) $cache_duration;
      $response->setMaxAge($cache_seconds);
      $response->setSharedMaxAge($cache_seconds);
    }

    return $response;
  }
}
